#include <stdio.h>
#include <stdlib.h>

#include "ptxlist.h"

#define CHECK if (!t) \
{ \
	printf ("FAILED\n"); \
	return EXIT_FAILURE; \
} \
else printf ("OK\n")

struct data_elem
{
	char *s;
	int nr;
};

void print_data (ptxtree *t)
{
	struct data_elem *tmp;
	
	printf ("print data: ");
	tmp = (struct data_elem *) get_tree_element_data (t);
	if (!tmp)
	{
		printf ("FAILED\n");
		exit (EXIT_FAILURE);
	}
	
	printf ("data->nr = %d\n", tmp->nr);
}

void print_tree_r (ptxtree *t, int i)
{
	int k;
	if (!t) return;
	for (k = i; k; k--) printf ("+");
	print_data (t);
	k = i;
	print_tree_r (t->down, ++i);
	print_tree_r (t->next, k);
}

void print_tree (ptxtree *t)
{
	goto_root (&t);
	goto_first_sibling (&t);
	
	print_tree_r (t, 0);
}

int main (void)
{
	int i;
	ptxtree *t;
	struct data_elem data;
	char *data_str = "0123456789abcdef";
	
	data.s = data_str;
	data.nr = 0;
	
	printf ("Init tree: ");
	t = init_tree ((void *) &data, sizeof (data));
	
	CHECK;
	
	printf ("Add a child: ");
	data.nr = 1;
	t = add_child (t, (void *) &data, sizeof (data));
		
	CHECK;
	
	printf ("Add 2 siblings: ");
	for (i = 2; i < 4; i++)
	{
		data.nr = i;
		t = add_sibling (t, (void *) &data, sizeof (data));
		if (!t) break;
	}
	CHECK;
	
	printf ("Add a child: ");
	data.nr = 4;
	t = add_child (t, (void *) &data, sizeof (data));
	
	CHECK;
	
	printf ("Add 2 siblings: ");
	for (i = 5; i < 7; i++)
	{
		data.nr = i;
		t = add_sibling (t, (void *) &data, sizeof (data));
		if (!t) break;
	}
	CHECK;
			
	printf ("Goto root: ");
	goto_root (&t);
	CHECK;
	print_data (t);

	printf ("Goto child: ");
	goto_child (&t);
	CHECK;
	print_data (t);
	
	printf ("Goto first sibling:");
	goto_first_sibling (&t);
	CHECK;
	print_data (t);
	
	printf ("Goto next sibling:");
	goto_next_sibling (&t);
	CHECK;
	print_data (t);
	
	printf ("Goto last sibling:");
	goto_last_sibling (&t);
	CHECK;
	print_data (t);
	
	printf ("Goto first child: ");
	goto_first_child (&t);
	CHECK;
	print_data (t);
	
	printf ("Goto parent: ");
	goto_parent (&t);
	CHECK;
	print_data (t);

	printf ("Goto last child: ");
	goto_last_child (&t);
	CHECK;
	print_data (t);
	
	printf ("Goto prev sibling: ");
	goto_prev_sibling (&t);
	CHECK;
	print_data (t);
	
	printf ("Free Tree: ");
	
	del_tree (t);
	printf ("OK\n");
	
	return EXIT_SUCCESS;
}
